using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;

namespace Feel
{
    public class CustomScooter : MVRScript
    {
        #region Vars    

        protected bool first_loading = false;

        protected JSONStorableFloat HandleRotation;
        protected JSONStorableFloat WheelSpeed;
        protected JSONStorableBool SpeedBooster;
        protected JSONStorableBool HeadLightComponents;
        protected JSONStorableFloat HeadLightValueComponents;
        protected JSONStorableBool MidLightComponents;
        protected JSONStorableFloat MidLightValueComponents;
        protected JSONStorableBool SignalLightComponents;
        protected JSONStorableBool PannelLightComponents;

        private float previousHandle;

        private Transform ObjectRoot;
        private Transform HandleRoot;
        private Transform HeadLightRoot;
        private Transform HeadLightHaloRoot;
        private Transform HeadLightEfRoot;
        private Transform SignalLightEfRoot;
        private Transform SideLight0Root;
        private Transform SideLight1Root;
        private Transform SideLight2Root;
        private Transform SideLight3Root;
        private Transform PannelLightEfRoot;
        private Transform PannelLightRoot;
        private Transform MidPlaneRoot;
        private Transform MidLightRoot;
        private Transform MidLightHaloRoot;
        private Transform FrontWheelRoot;
        private Transform MidWheelRoot;


        private bool editorInitialized = false;
        private int initLoops = 0;
        private int maxInitLoops = 1500; // 3600 for one minute

        private CustomScooterColor _scooter_color;

        private Transform MainColorRoot;
        private Transform CapColorRoot;

        #endregion

        public override void Init()
        {
            try
            {
                if (containingAtom.type != "CustomUnityAsset")
                {
                    SuperController.LogError("CustomScooter - Please add this script on a Custom Unity Asset ( script disabled).");
                    return;
                }

                HandleRotation = new JSONStorableFloat("Handle Rotation", 0f, OnHandleChanged, -1f, 1f);
                RegisterFloat(HandleRotation);
                CreateSlider(HandleRotation, false);

                WheelSpeed = new JSONStorableFloat("Wheel Speed", 0f, OnWheelSpeedUpdate, -1f, 1f);
                RegisterFloat(WheelSpeed);
                CreateSlider(WheelSpeed, false);

                SpeedBooster = new JSONStorableBool("SpeedBooster", false, OnSpeedBooster);
                RegisterBool(SpeedBooster);
                UIDynamicToggle boostertoggle = CreateToggle(SpeedBooster, false);
                boostertoggle.label = "Booster On Off";

                HeadLightValueComponents = new JSONStorableFloat("Head Light Value", 1f, OnHeadLightValueUpdate, 0f, 1f);
                RegisterFloat(HeadLightValueComponents);
                CreateSlider(HeadLightValueComponents, false);

                MidLightValueComponents = new JSONStorableFloat("Mid Light Value", 1f, OnMidLightValueUpdate, 0f, 1f);
                RegisterFloat(MidLightValueComponents);
                CreateSlider(MidLightValueComponents, false);

                HeadLightComponents = new JSONStorableBool("HeadLightComponents", false, OnHeadLightToggle);
                RegisterBool(HeadLightComponents);
                UIDynamicToggle headlighttoggle = CreateToggle(HeadLightComponents, false);
                headlighttoggle.label = "Head Light On Off";

                MidLightComponents = new JSONStorableBool("MidLightComponents", false, OnMidLightToggle);
                RegisterBool(MidLightComponents);
                UIDynamicToggle midlighttoggle = CreateToggle(MidLightComponents, false);
                midlighttoggle.label = "Mid Light On Off";

                SignalLightComponents = new JSONStorableBool("SignalLightComponents", false, OnSignalLightToggle);
                RegisterBool(SignalLightComponents);
                UIDynamicToggle signallighttoggle = CreateToggle(SignalLightComponents, false);
                signallighttoggle.label = "Signal Light On Off";

                PannelLightComponents = new JSONStorableBool("PannelLightComponents", false, OnPannelLightToggle);
                RegisterBool(PannelLightComponents);
                UIDynamicToggle pannellighttoggle = CreateToggle(PannelLightComponents, false);
                pannellighttoggle.label = "Pannel Light On Off";

                _scooter_color = new CustomScooterColor();
                _scooter_color.Init(this);

                // A tiny bit of help to give some info to your fellow creator and/or player
                JSONStorableString helpText = new JSONStorableString("About",
                    "<color=#000><size=35><b>Custom Scooter</b></size></color>\n\n" +
                    "<color=#333>" +
                    "store the position of the target,\n" +
                    "or Track a specific target.\n\n" +
                    "thank you for your support\n" +
                    "<b>https://www.patreon.com/feel_vam</b>\n\n" +
                    "</color>");
                UIDynamic helpTextfield = CreateTextField(helpText, true);
                helpTextfield.height = 300.0f;

                var btn = CreateButton("Go To Support", true);
                btn.button.onClick.AddListener(() =>
                {
                    Application.OpenURL("https://www.patreon.com/feel_vam");
                });

            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        // Update is called with each rendered frame by Unity
        void Update()
        {
            // I don't know why the fuck this asset does not get the children properly during the loading process, so I'm checking the singleton
            // In reality, that's even cleaner to do it that way
            if (initLoops < maxInitLoops && editorInitialized == false && !SuperController.singleton.isLoading)
            {
                initLoops++;
                InitEditor();
            }

            if (initLoops == maxInitLoops)
            {
                SuperController.LogError("CustomScooter : This CUA could not be initialized properly. Please select a compatible object and reload the script.");
                initLoops++; // To avoid spamming the log with the error
            }

            // first set.
            if(first_loading == false && HandleRoot != null && HeadLightRoot != null && MidLightRoot != null && SignalLightEfRoot != null)
            {
                //SuperController.LogMessage(">> " + LiquidTransComponents.val);
                //SuperController.LogMessage(">> " + GlassTransComponents.val);
                //SuperController.LogMessage(">> ----------------------------");
                first_loading = true;
                OnHeadLightToggle(HeadLightComponents);
                OnMidLightToggle(MidLightComponents);
                OnSignalLightToggle(SignalLightComponents);
                OnPannelLightToggle(PannelLightComponents);
            }

            if(FrontWheelRoot && MidWheelRoot)
            {
                float b = 1f;
                if (SpeedBooster.val) b = 2f;
                float a = 0.5f * (WheelSpeed.val * b);
                FrontWheelRoot.Rotate(Vector3.left * a * -45);
                MidWheelRoot.Rotate(Vector3.left * a * -45);
            }
        }

        void OnDestroy()
        {
            RemoveToggle(HeadLightComponents);
            RemoveToggle(MidLightComponents);
            RemoveToggle(SignalLightComponents);
            RemoveToggle(PannelLightComponents);
        }

        private bool InitEditor()
        {
            if (editorInitialized == true) return true;

            ObjectRoot = getObjectRoot(containingAtom.reParentObject, "custom_scooter");
            
            if (ObjectRoot == null)
            {
                return false;
            }

            HandleRoot = getChildRoot(containingAtom.reParentObject, "FirstWheel");
            HeadLightRoot = getChildRoot(containingAtom.reParentObject, "HeadLight");
            HeadLightHaloRoot = getChildRoot(containingAtom.reParentObject, "HeadLightHalo");
            HeadLightEfRoot = getChildRoot(containingAtom.reParentObject, "HeadLightEf");
            SignalLightEfRoot = getChildRoot(containingAtom.reParentObject, "SignalLightEf");
            SideLight0Root = getChildRoot(containingAtom.reParentObject, "SideLight0");
            SideLight1Root = getChildRoot(containingAtom.reParentObject, "SideLight1");
            SideLight2Root = getChildRoot(containingAtom.reParentObject, "SideLight2");
            SideLight3Root = getChildRoot(containingAtom.reParentObject, "SideLight3");

            PannelLightEfRoot = getChildRoot(containingAtom.reParentObject, "SideLight3");
            PannelLightRoot = getChildRoot(containingAtom.reParentObject, "PannelHeadLight");
            MidLightRoot = getChildRoot(containingAtom.reParentObject, "MidLight");
            MidLightHaloRoot = getChildRoot(containingAtom.reParentObject, "MidLightHalo");
            MidPlaneRoot = getChildRoot(containingAtom.reParentObject, "MidLightEf");

            FrontWheelRoot = getChildRoot(containingAtom.reParentObject, "FrontWheel");
            MidWheelRoot = getChildRoot(containingAtom.reParentObject, "MidWheel");

            MainColorRoot = getChildRoot(containingAtom.reParentObject, "Main Body");
            CapColorRoot = HandleRoot.Find("Wheel_Cap");

            _scooter_color.SetRoot(MainColorRoot, CapColorRoot);

            // I found my transforms, now the rest
            
            if (HandleRoot && HeadLightRoot && HeadLightEfRoot && SignalLightEfRoot && SideLight0Root && SideLight1Root && SideLight2Root && SideLight3Root && MainColorRoot)
            {
                editorInitialized = true;

                // Everything is initialized, triggering the callbacks to init the door rotation saved in the scene file
                // I'm just quickly fixing the script for VamTastic, I'm not gonna refactor everything, so in reality,
                // the float val param of the callbacks is useless
                OnHandleChanged(HandleRotation.val);
                _scooter_color.UpdateValue();


                return true;
            }
            else
            {
                SuperController.LogError("CustomScooter : your CUA is missing mandatory components.");
                initLoops = maxInitLoops; // We don't have what we need, we consider the initialization failed
                return false;
            }

        }

        // Recursive function searching for a transform containing the string set in objectNameSearchString
        // It is used the first time for the initialization of the script
        // This script NEEDS to use a string "containing" your original because the cloning of the original asset will add "Clone" behind it.
        private Transform getObjectRoot(Transform parent, string rootName)
        {
            foreach (Transform child in parent)
            {
                if (child.name.Contains(rootName))
                {
                    return child;
                }
                else
                {
                    Transform childSearch = getObjectRoot(child, rootName);
                    if (childSearch != null) return childSearch;
                }
            }

            return null;
        }

        private Transform getChildRoot(Transform parent, string searchName)
        {
            foreach (Transform child in parent)
            {
                if (child.name == searchName)
                {

                    return child;
                }
                else
                {
                    Transform childSearch = getChildRoot(child, searchName);
                    if (childSearch != null) return childSearch;
                }
            }

            return null;
        }

        void OnHandleChanged(float rotation)
        {
            {
                if (HandleRoot == null) return;

                var Slider1 = HandleRotation.val;
                var a = Slider1 - previousHandle;
                HandleRoot.transform.Rotate(Vector3.up * a * -45, Space.Self);
                previousHandle = HandleRotation.val;
            }
        }

        void OnWheelSpeedUpdate(float speed)
        {

        }

        void OnHeadLightValueUpdate(float value)
        {
            if (HeadLightRoot == null) return;
            if (HeadLightHaloRoot == null) return;
            if (HeadLightEfRoot == null) return;

            Light light = HeadLightRoot.GetComponent<Light>();
            light.intensity = value * 2f;

            if(value >= 0.5f)
                HeadLightHaloRoot.gameObject.SetActive(true);
            else
                HeadLightHaloRoot.gameObject.SetActive(false);
        }

        void OnMidLightValueUpdate(float value)
        {
            if (MidLightRoot == null) return;
            if (MidLightHaloRoot == null) return;
            if (MidPlaneRoot == null) return;

            Light light = MidLightRoot.GetComponent<Light>();
            light.intensity = value * 1.5f;

            if (value >= 0.5f)
                MidLightHaloRoot.gameObject.SetActive(true);
            else
                MidLightHaloRoot.gameObject.SetActive(false);
        }

        void OnSpeedBooster(JSONStorableBool jb)
        {

        }

        void OnHeadLightToggle(JSONStorableBool jb)
        {
            if (HeadLightRoot == null) return;
            if (HeadLightHaloRoot == null) return;
            if (HeadLightEfRoot == null) return;

            if (jb.val)
            {
                HeadLightRoot.gameObject.SetActive(true);
                HeadLightHaloRoot.gameObject.SetActive(true);
                HeadLightEfRoot.gameObject.SetActive(true);
            }
            else
            {
                HeadLightRoot.gameObject.SetActive(false);
                HeadLightHaloRoot.gameObject.SetActive(false);
                HeadLightEfRoot.gameObject.SetActive(false);
            }
        }

        void OnMidLightToggle(JSONStorableBool jb)
        {
            if (MidLightRoot == null) return;
            if (MidLightHaloRoot == null) return;
            if (MidPlaneRoot == null) return;

            if (jb.val)
            {
                MidLightRoot.gameObject.SetActive(true);
                MidLightHaloRoot.gameObject.SetActive(true);
                MidPlaneRoot.gameObject.SetActive(true);
            }
            else
            {
                MidLightRoot.gameObject.SetActive(false);
                MidLightHaloRoot.gameObject.SetActive(false);
                MidPlaneRoot.gameObject.SetActive(false);
            }
        }

        void OnSignalLightToggle(JSONStorableBool jb)
        {
            if (SignalLightEfRoot == null) return;
            if (SideLight0Root == null) return;
            if (SideLight1Root == null) return;
            if (SideLight2Root == null) return;
            if (SideLight3Root == null) return;

            if (jb.val)
            {
                SignalLightEfRoot.gameObject.SetActive(true);
                SideLight0Root.gameObject.SetActive(true);
                SideLight1Root.gameObject.SetActive(true);
                SideLight2Root.gameObject.SetActive(true);
                SideLight3Root.gameObject.SetActive(true);
            }
            else
            {
                SignalLightEfRoot.gameObject.SetActive(false);
                SideLight0Root.gameObject.SetActive(false);
                SideLight1Root.gameObject.SetActive(false);
                SideLight2Root.gameObject.SetActive(false);
                SideLight3Root.gameObject.SetActive(false);
            }
        }

        void OnPannelLightToggle(JSONStorableBool jb)
        {
            if (PannelLightRoot == null) return;
            if (PannelLightEfRoot == null) return;

            if (jb.val)
            {
                PannelLightRoot.gameObject.SetActive(true);
                PannelLightEfRoot.gameObject.SetActive(true);
            }
            else
            {
                PannelLightRoot.gameObject.SetActive(false);
                PannelLightEfRoot.gameObject.SetActive(false);
            }
        }

    }

}