using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;

public class CustomScooterColor
{

    private MVRScript _root;
    private Transform _MainRoot;
    private Transform _CapRoot;

    protected JSONStorableColor MainColor;
    protected JSONStorableColor PointColor;
    protected JSONStorableColor SeatColor;
    protected JSONStorableColor LineColor;

    private Material MainMaterial;
    private Material CapMainMaterial;
    private Material PointMaterial;
    private Material SeatMaterial;
    private Material LineMaterial;

    protected List<String> _colorPatterns;
    protected JSONStorableStringChooser _colorPatternsJSON;
    protected UIDynamicPopup _displayPopup;

    public void Init(MVRScript root)
    {
        _root = root;

        HSVColor hsvc1 = HSVColorPicker.RGBToHSV(0.878f, 0.878f, 0.878f);
        MainColor = new JSONStorableColor("Main Color", hsvc1, MainColorCallback);
        _root.RegisterColor(MainColor);
        _root.CreateColorPicker(MainColor, true);

        HSVColor hsvc2 = HSVColorPicker.RGBToHSV(0.278f, 0.674f, 0.647f);
        PointColor = new JSONStorableColor("Point Color", hsvc2, PointColorCallback);
        _root.RegisterColor(PointColor);
        _root.CreateColorPicker(PointColor, true);

        HSVColor hsvc3 = HSVColorPicker.RGBToHSV(0.549f, 0.286f, 0.2f);
        SeatColor = new JSONStorableColor("Seat Color", hsvc3, SeatColorCallback);
        _root.RegisterColor(SeatColor);
        _root.CreateColorPicker(SeatColor, true);

        HSVColor hsvc4 = HSVColorPicker.RGBToHSV(0.745f, 0.745f, 0.745f);
        LineColor = new JSONStorableColor("LIne Color", hsvc4, LineColorCallback);
        _root.RegisterColor(LineColor);
        _root.CreateColorPicker(LineColor, true);

        _colorPatterns = new List<string>();
        _colorPatterns.Add("Green");
        _colorPatterns.Add("Red");
        _colorPatterns.Add("Black");
        _colorPatterns.Add("White");
        _colorPatterns.Add("Sky");

        _colorPatternsJSON = new JSONStorableStringChooser("Pattern Choice", null, "Custom", "", OnColorSelect);
        _colorPatternsJSON.storeType = JSONStorableParam.StoreType.Physical;
        _root.RegisterStringChooser(_colorPatternsJSON);

        _displayPopup = _root.CreateScrollablePopup(_colorPatternsJSON, false);
        _displayPopup.popupPanelHeight = 300f;
        _displayPopup.popup.onOpenPopupHandlers += SyncColorPatternChoices;
        _displayPopup.label = "Pattern Type:";

        _colorPatternsJSON.choices = _colorPatterns;

    }

    protected void OnColorSelect(string colorType)
    {
        //SuperController.LogMessage(">> OrificeSelect " + orificeType);
        switch (colorType)
        {
            case "Red":
                MainColor.val = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
                PointColor.val = HSVColorPicker.RGBToHSV(0.45f, 0f, 0f);
                SeatColor.val = HSVColorPicker.RGBToHSV(0.549f, 0.286f, 0.2f);
                LineColor.val = HSVColorPicker.RGBToHSV(0.45f, 0f, 0f);
                break;
            case "Black":
                MainColor.val = HSVColorPicker.RGBToHSV(0f, 0f, 0f);
                PointColor.val = HSVColorPicker.RGBToHSV(0f, 0.1f, 0.1f);
                SeatColor.val = HSVColorPicker.RGBToHSV(0.141f, 0.141f, 0.141f);
                LineColor.val = HSVColorPicker.RGBToHSV(0.627f, 0.627f, 0.627f);
                break;
            case "White":
                MainColor.val = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
                PointColor.val = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
                SeatColor.val = HSVColorPicker.RGBToHSV(0.549f, 0.286f, 0.2f);
                LineColor.val = HSVColorPicker.RGBToHSV(0.627f, 0.627f, 0.627f);
                break;
            case "Sky":
                MainColor.val = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
                PointColor.val = HSVColorPicker.RGBToHSV(0.537f, 0.812f, 1f);
                SeatColor.val = HSVColorPicker.RGBToHSV(0.517f, 0.517f, 0.517f);
                LineColor.val = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
                break;
            default:
                MainColor.val = HSVColorPicker.RGBToHSV(0.878f, 0.878f, 0.878f);
                PointColor.val = HSVColorPicker.RGBToHSV(0.278f, 0.674f, 0.647f);
                SeatColor.val = HSVColorPicker.RGBToHSV(0.549f, 0.286f, 0.2f);
                LineColor.val = HSVColorPicker.RGBToHSV(0.745f, 0.745f, 0.745f);

                break;
        }
        UpdateValue();
    }

    public void UpdateValue()
    {
        MainColorCallback(MainColor);
        PointColorCallback(PointColor);
        SeatColorCallback(SeatColor);
        LineColorCallback(LineColor);
    }

    protected void SyncColorPatternChoices()
    {
        //SuperController.LogMessage(">> SyncOrificeChoices ");
        //UpdateOrificeChoices();
    }

    public void SetRoot(Transform main_root, Transform cap_root)
    {
        _MainRoot = main_root;
        _CapRoot = cap_root;

        Renderer renderer = _MainRoot.GetComponent<Renderer>();
        Material[] list1 = renderer.materials;

        MainMaterial = list1[1];
        PointMaterial = list1[2];
        SeatMaterial = list1[3];
        LineMaterial = list1[4];

        Renderer renderer2 = _CapRoot.GetComponent<Renderer>();
        Material[] list2 = renderer2.materials;

        CapMainMaterial = list2[1];

        //SuperController.LogMessage(">> " + CapMainMaterial);
    }

    public void MainColorCallback(JSONStorableColor dc)
    {
        if (_MainRoot == null) return;

        Color c = Color.HSVToRGB(dc.valNoCallback.H, dc.valNoCallback.S, dc.valNoCallback.V);

        MainMaterial.SetColor("_Color", c);
        
    }

    public void PointColorCallback(JSONStorableColor dc)
    {
        if (_MainRoot == null) return;

        Color c = Color.HSVToRGB(dc.valNoCallback.H, dc.valNoCallback.S, dc.valNoCallback.V);

        PointMaterial.SetColor("_Color", c);
        CapMainMaterial.SetColor("_Color", c);
    }

    public void SeatColorCallback(JSONStorableColor dc)
    {
        if (_MainRoot == null) return;

        Color c = Color.HSVToRGB(dc.valNoCallback.H, dc.valNoCallback.S, dc.valNoCallback.V);

        SeatMaterial.SetColor("_Color", c);

    }

    public void LineColorCallback(JSONStorableColor dc)
    {
        if (_MainRoot == null) return;

        Color c = Color.HSVToRGB(dc.valNoCallback.H, dc.valNoCallback.S, dc.valNoCallback.V);

        LineMaterial.SetColor("_Color", c);

    }
}